@@ -11,7 +11,7 @@
 #      modify it under the terms of Perl's Artistic License.        #
 #                                                                   #
 #####################################################################
-# $Id: IRC.pm,v 1.3 1999/08/12 21:07:08 corbeau Exp $
+# $Id: IRC.pm,v 1.10 2004/04/30 18:02:51 jmuhlich Exp $
 
 
 package Net::IRC;
@@ -19,33 +19,59 @@ package Net::IRC;
 BEGIN { require 5.004; }    # needs IO::* and $coderef->(@args) syntax 
 
 use Net::IRC::Connection;
+use Net::IRC::EventQueue;
 use IO::Select;
 use Carp;
+
+
+# grab the drop-in replacement for time() from Time::HiRes, if it's available
+BEGIN {
+   Time::HiRes->import('time') if eval "require Time::HiRes";
+}
+
+
 use strict;
 use vars qw($VERSION);
 
-$VERSION = "0.73";
+$VERSION = "0.75";
 
+sub new {
+  my $proto = shift;
+  
+  my $self = {
+    '_conn'             => [],
+    '_connhash'         => {},
+    '_error'            => IO::Select->new(),
+    '_debug'            => 0,
+    '_schedulequeue'    => new Net::IRC::EventQueue(),
+    '_outputqueue'      => new Net::IRC::EventQueue(),
+    '_read'             => IO::Select->new(),
+    '_timeout'          => 1,
+    '_write'            => IO::Select->new(),
+  };
+  
+  bless $self, $proto;
+  
+  return $self;
+}
 
+sub outputqueue {
+  my $self = shift;
+  return $self->{_outputqueue};
+}
 
-
-# -- #perl was here --
-# <Addi> purl, karma c?
-# <purl> c has karma of 666
-
-
-#####################################################################
-#        Methods start here, arranged in alphabetical order.        #
-#####################################################################
-
+sub schedulequeue {
+  my $self = shift;
+  return $self->{_schedulequeue};
+}
 
 # Front end to addfh(), below. Sets it to read by default.
 # Takes at least 1 arg:  an object to add to the select loop.
 #           (optional)   a flag string to pass to addfh() (see below)
 sub addconn {
-    my ($self, $conn) = @_;
-    
-    $self->addfh( $conn->socket, $conn->can('parse'), ($_[2] || 'r'), $conn);
+  my ($self, $conn) = @_;
+  
+  $self->addfh( $conn->socket, $conn->can('parse'), ($_[2] || 'r'), $conn);
 }
 
 # Adds a filehandle to the select loop. Tasty and flavorful.
@@ -56,254 +82,192 @@ sub addconn {
 #                  except that you can combine flags (i.e., "rw").
 #    (optional)  an object that the coderef is a method of
 sub addfh {
-    my ($self, $fh, $code, $flag, $obj) = @_;
-    my ($letter);
-
-    die "Not enough arguments to IRC->addfh()" unless defined $code;
-
-    # -- #perl was here! --
-    # *** sonic_1 (rick@highlife.net) has joined channel #perl
-    # <sonic_1> can someone tell me whats wrong with this script?
-    # <sonic_1> #!/usr/bin/perl
-    # <sonic_1> for $x in 'ls --width=1';
-    # <sonic_1> do 'mv $x ~$x/Mailbox;chown $x.mail ~$x/Mailbox';
-    # <sonic_1> fi
-    
-    if ($flag) {
-	foreach $letter (split(//, lc $flag)) {
-	    if ($letter eq 'r') {
-		$self->{_read}->add( $fh );
-	    } elsif ($letter eq 'w') {
-		$self->{_write}->add( $fh );
-	    } elsif ($letter eq 'e') {
-		$self->{_error}->add( $fh );
-	    }
-	}
-    } else {
-	$self->{_read}->add( $fh );
+  my ($self, $fh, $code, $flag, $obj) = @_;
+  my ($letter);
+  
+  die "Not enough arguments to IRC->addfh()" unless defined $code;
+  
+  if ($flag) {
+    foreach $letter (split(//, lc $flag)) {
+      if ($letter eq 'r') {
+        $self->{_read}->add( $fh );
+      } elsif ($letter eq 'w') {
+        $self->{_write}->add( $fh );
+      } elsif ($letter eq 'e') {
+        $self->{_error}->add( $fh );
+      }
     }
-    
-    $self->{_connhash}->{$fh} = [ $code, $obj ];
+  } else {
+    $self->{_read}->add( $fh );
+  }
+  
+  $self->{_connhash}->{$fh} = [ $code, $obj ];
 }
 
-
 # Sets or returns the debugging flag for this object.
 # Takes 1 optional arg: a new boolean value for the flag.
 sub debug {
-    my $self = shift;
-
-    if (@_) {
-	$self->{_debug} = $_[0];
-    }
-    return $self->{_debug};
+  my $self = shift;
+  
+  if (@_) {
+    $self->{_debug} = $_[0];
+  }
+  return $self->{_debug};
 }
 
-
 # Goes through one iteration of the main event loop. Useful for integrating
 # other event-based systems (Tk, etc.) with Net::IRC.
 # Takes no args.
 sub do_one_loop {
-    my $self = shift;
-    
-    # -- #perl was here! --
-    # <ChipDude> Pudge:  Do not become the enemy.
-    #   <^Pudge> give in to the dark side, you knob.
-    
-    my ($ev, $sock, $time, $nexttimer, $timeout);
-    
-    # Check the queue for scheduled events to run.
-    
-    $time = time();             # no use calling time() all the time.
-    $nexttimer = 0;
-    foreach $ev ($self->queue) {
-	if ($self->{_queue}->{$ev}->[0] <= $time) {
-	    $self->{_queue}->{$ev}->[1]->
-		(@{$self->{_queue}->{$ev}}[2..$#{$self->{_queue}->{$ev}}]);
-	    delete $self->{_queue}->{$ev};
-	} else {
-	    $nexttimer = $self->{_queue}->{$ev}->[0] 
-		if ($self->{_queue}->{$ev}->[0] < $nexttimer
-		    or not $nexttimer);
-	}
-    }
-    
-    # Block until input arrives, then hand the filehandle over to the
-    # user-supplied coderef. Look! It's a freezer full of government cheese!
+  my $self = shift;
+  my ($ev, $sock, $time, $nexttimer, $timeout);
+  my (undef, undef, undef, $caller) = caller(1);
 
-    if ($nexttimer) {
-	$timeout = $nexttimer - $time < $self->{_timeout}
-	? $nexttimer - $time : $self->{_timeout};
-    } else {
-	$timeout = $self->{_timeout};
+  $time = time();             # no use calling time() all the time.
+
+  if(!$self->outputqueue->is_empty) {
+    my $outputevent = undef;
+    while(defined($outputevent = $self->outputqueue->head)
+          && $outputevent->time <= $time) {
+      $outputevent = $self->outputqueue->dequeue();
+      $outputevent->content->{coderef}->(@{$outputevent->content->{args}});
     }
-    foreach $ev (IO::Select->select($self->{_read},
-				    $self->{_write},
-				    $self->{_error},
-				    $timeout)) {
-	foreach $sock (@{$ev}) {
-	    my $conn = $self->{_connhash}->{$sock};
-	    
-	    # $conn->[0] is a code reference to a handler sub.
-	    # $conn->[1] is optionally an object which the
-	    #    handler sub may be a method of.
-	    
-	    $conn->[0]->($conn->[1] ? ($conn->[1], $sock) : $sock);
-	}
+    $nexttimer = $self->outputqueue->head->time if !$self->outputqueue->is_empty();
+  }
+
+  # we don't want to bother waiting on input or running
+  # scheduled events if we're just flushing the output queue
+  # so we bail out here
+  return if $caller eq 'Net::IRC::flush_output_queue';
+
+  # Check the queue for scheduled events to run.
+  if(!$self->schedulequeue->is_empty) {
+    my $scheduledevent = undef;
+    while(defined($scheduledevent = $self->schedulequeue->head) && $scheduledevent->time <= $time) {
+      $scheduledevent = $self->schedulequeue->dequeue();
+      $scheduledevent->content->{coderef}->(@{$scheduledevent->content->{args}});
     }
+    if(!$self->schedulequeue->is_empty()
+       && $nexttimer
+       && $self->schedulequeue->head->time < $nexttimer) {
+      $nexttimer = $self->schedulequeue->head->time;
+    }
+  }
+
+  # Block until input arrives, then hand the filehandle over to the
+  # user-supplied coderef. Look! It's a freezer full of government cheese!
+  
+  if ($nexttimer) {
+    $timeout = $nexttimer - $time < $self->{_timeout}
+    ? $nexttimer - $time : $self->{_timeout};
+  } else {
+    $timeout = $self->{_timeout};
+  }
+  foreach $ev (IO::Select->select($self->{_read},
+                                  $self->{_write},
+                                  $self->{_error},
+                                  $timeout)) {
+    foreach $sock (@{$ev}) {
+      my $conn = $self->{_connhash}->{$sock};
+      $conn or next;
+    
+      # $conn->[0] is a code reference to a handler sub.
+      # $conn->[1] is optionally an object which the
+      #    handler sub may be a method of.
+      
+      $conn->[0]->($conn->[1] ? ($conn->[1], $sock) : $sock);
+    }
+  }
 }
 
-# Ye Olde Contructor Methode. You know the drill.
-# Takes absolutely no args whatsoever.
-sub new {
-    my $proto = shift;
+sub flush_output_queue {
+  my $self = shift;
 
-    # -- #perl was here! --
-    # *** Notice -- Received KILL message for `akira`. From billn  Path:
-    # *.concentric.net[irc@ircd.concentric.net]!*.umn.edu[irc2.tc.umn.edu]!
-    # irc-phx.primenet.com[irc@irc06.primenet.com]!usr05.primenet.com!billn
-    # (User requested. Wants to be a spambot.)
-    
-    my $self = {
-	        '_conn'     => [],
-		'_connhash' => {},
-		'_error'    => IO::Select->new(),
-		'_debug'    => 0,
-		'_queue'    => {},
-		'_qid'      => 'a',
-		'_read'     => IO::Select->new(),
-		'_timeout'  => 1,
-		'_write'    => IO::Select->new(),
-	    };
-
-    bless $self, $proto;
-
-    # -- #perl was here! --
-    # *** Notice -- Received KILL message for [Gump]. From TrueCynic Path: *.
-    # concentric.net[irc@ircd.concentric.net]!irc.best.net!usr10.primenet.com
-    # !TrueCynic (Clone, Forrest! Clone!)
-    
-    return $self;
+  while(!$self->outputqueue->is_empty()) {
+    $self->do_one_loop();
+  }
 }
 
 # Creates and returns a new Connection object.
 # Any args here get passed to Connection->connect().
 sub newconn {
-    my $self = shift;
-    my $conn = Net::IRC::Connection->new($self, @_);
-    
-    return if $conn->error;
-    return $conn;
+  my $self = shift;
+  my $conn = Net::IRC::Connection->new($self, @_);
+  
+  return if $conn->error;
+  return $conn;
 }
 
-
-# -- #perl was here! --
-# <_thoth> occurrence
-# <_thoth> Ranking right up there with "separate", the word "occur" and
-#          its compounds are amongst the most commonly misspelled words
-#          in the whole English language.
-# <_thoth> First of all, "occur" in English derives from a Latin verb
-#          (occurrere) whose conjugation happens to be something other
-#          than the 1st (-are verbs), and 1st conjugation verbs are the
-#          *ONLY* Latin ones that go to -ance and -able compounds in English.
-#          (German ones also go to the -a- forms.).
-# <_thoth> Thus, it must form compounds with -ence and -ible.  If you can't
-#          remember any Latin, it's simple to think of what it would be in
-#          any modern Latin dialect, like Spanish, French, Italian, or
-#          Portuguese.  In all cases, you have a verb ending in /[ei]re?$/,
-#          (in fact, /^occ?o?urr?[ei]re?$/ describes them all), which clues
-#          you in that it's not first conjugation at all.  That means -ance
-#          is dead wrong.
-# nvp mutters something.  could it be: "holy fuck, thoth"
-# <_thoth> Secondly, English verbs stressed on the last syllable using "short"
-#          vowels in English (like infer, beget, forget, and incur) always
-#          double their final consonant when adding suffixes, thus providing
-#          us with infeRRing and infeRRIBLE, begeTTing and begaTTer,
-#          forgeTTing and forgeTTABLE, and incuRRed, incuRRing, and
-#          incuRRENCE. Thus it must be occuRRed, occuRRing, and occuRRENCE.
-# <_thoth> Except that sometimes this is wrong.
-# <_thoth> TorgoX has a paper on it.
-
-
-
-# Returns a list of listrefs to event scheduled to be run.
 # Takes the args passed to it by Connection->schedule()... see it for details.
-sub queue {
-    my $self = shift;
+sub enqueue_scheduled_event {
+  my $self = shift;
+  my $time = shift;
+  my $coderef = shift;
+  my @args = @_;
 
-    if (@_) {
-        $self->{_qid} = 'a' if $self->{_qid} eq 'zzzzzzzz';
-        my $id = $self->{_qid};
-        $self->{_queue}->{$self->{_qid}++} = [ @_ ];
-        return ($id);
-
-    } else {
-        return keys %{$self->{_queue}};
-    }
+  return $self->schedulequeue->enqueue($time, { coderef => $coderef, args => \@args });
 }
 
-
 # Takes a scheduled event ID to remove from the queue.
 # Returns the deleted coderef, if you actually care.
-sub dequeue {
-    my ($self, $id) = @_;
-    delete $self->{_queue}->{$id}
+sub dequeue_scheduled_event {
+  my ($self, $id) = @_;
+  $self->schedulequeue->dequeue($id);
 }
 
+# Takes the args passed to it by Connection->schedule()... see it for details.
+sub enqueue_output_event {
+  my $self = shift;
+  my $time = shift;
+  my $coderef = shift;
+  my @args = @_;
 
-# -- #perl was here! --
-#   <Tkil> chaos_ -- oh... and if you're a beginner... don't worry overmuch
-#          about efficiency.
-#  <billn> if I wanted efficency, I'd put bigger tires on my truck, so I could
-#          get the whole cat in one pass.
+  return $self->outputqueue->enqueue($time, { coderef => $coderef, args => \@args });
+}
+
+# Takes a scheduled event ID to remove from the queue.
+# Returns the deleted coderef, if you actually care.
+sub dequeue_output_event {
+  my ($self, $id) = @_;
+  $self->outputqueue->dequeue($id);
+}
 
 # Front-end for removefh(), below.
 # Takes 1 arg:  a Connection (or DCC or whatever) to remove.
 sub removeconn {
-    my ($self, $conn) = @_;
-    
-    $self->removefh( $conn->socket );
+  my ($self, $conn) = @_;
+  
+  $self->removefh( $conn->socket );
 }
 
 # Given a filehandle, removes it from all select lists. You get the picture.
 sub removefh {
-    my ($self, $fh) = @_;
-    
-    $self->{_read}->remove( $fh );
-    $self->{_write}->remove( $fh );
-    $self->{_error}->remove( $fh );
-    delete $self->{_connhash}->{$fh};
+  my ($self, $fh) = @_;
+  
+  $self->{_read}->remove( $fh );
+  $self->{_write}->remove( $fh );
+  $self->{_error}->remove( $fh );
+  delete $self->{_connhash}->{$fh};
 }
 
-# -- #perl was here! --
-#  <^Pudge>  orwant: was there a Perl before 5.004?
-#  <orwant>  ^Pudge: Nope, perl5.004 flew out of Chip's head like
-#            Athena out of Zeus.
-#  <oznoid>  heh!
-
 # Begin the main loop. Wheee. Hope you remembered to set up your handlers
 # first... (takes no args, of course)
 sub start {
-    my $self = shift;
-
-    while (1) {
-	$self->do_one_loop();
-    }
+  my $self = shift;
+  
+  while (1) {
+    $self->do_one_loop();
+  }
 }
 
 # Sets or returns the current timeout, in seconds, for the select loop.
 # Takes 1 optional arg:  the new value for the timeout, in seconds.
 # Fractional timeout values are just fine, as per the core select().
 sub timeout {
-    my $self = shift;
-
-    # -- #perl was here! --
-    # <fimmtiu>  asylum, then your disk is full. *shrug*  Wield rm -f wildly.
-    # <asylum_>  fimmtiu. yeah right dude.
-    #   <DemoN>  rm -f is too slow.  newfs.
-    # <rudefix>  hell, i just prefer to take the disk out and lick it!
-    
-    if (@_) { $self->{_timeout} = $_[0] }
-    return $self->{_timeout};
+  my $self = shift;
+  
+  if (@_) { $self->{_timeout} = $_[0] }
+  return $self->{_timeout};
 }
 
 1;
@@ -490,6 +454,13 @@ Password
 If the IRC server you're trying to write a bot for is
 password-protected, no problem. Just say "C<Password => 'foo'>" and
 you're set.
+
+=item *
+
+SSL
+
+If you wish to connect to an irc server which is using SSL, set this to a
+true value.  Ie: "C<SSL => 1>".
     
 =back
 
@@ -712,6 +683,17 @@ B<Optional:> A new value for the C<select()> timeout for this IRC object.
 
 =back
 
+=item *
+
+flush_output_queue()
+
+Flushes any waiting messages in the output queue if pacing is enabled. This
+method will not return until the output queue is empty.
+
+=over
+
+=back
+
 =head1 AUTHORS
 
 =over
@@ -737,7 +719,7 @@ http://www.execpc.com/~corbeau/irc/list.html .
 =head1 URL
 
 Up-to-date source and information about the Net::IRC project can be found at
-http://netirc.betterbox.net/ .
+http://www.sourceforge.net/projects/net-irc/ .
 
 =head1 SEE ALSO
 
